@@ -14,7 +14,6 @@ import (
 	"github.com/pkg/errors"
 
 	"kubevirt.io/containerized-data-importer/pkg/image"
-	"kubevirt.io/containerized-data-importer/pkg/util"
 )
 
 type fakeInfoOpRetVal struct {
@@ -258,36 +257,18 @@ var _ = Describe("Data Processor", func() {
 		})
 	})
 
-	const (
-		Mi              = int64(1024 * 1024)
-		Gi              = 1024 * Mi
-		noOverhead      = float64(0)
-		defaultOverhead = float64(0.055)
-		largeOverhead   = float64(0.75)
-	)
-	table.DescribeTable("getusablespace should return properly aligned sizes,", func(virtualSize int64, overhead float64) {
-		for i := int64(virtualSize - 1024); i < virtualSize+1024; i++ {
-			// Requested space is virtualSize rounded up to 1Mi alignment / (1 - overhead) rounded up
-			requestedSpace := int64(float64(util.RoundUp(i, util.DefaultAlignBlockSize)+1) / float64(1-overhead))
-			if i <= virtualSize {
-				Expect(GetUsableSpace(overhead, requestedSpace)).To(Equal(virtualSize))
-			} else {
-				Expect(GetUsableSpace(overhead, requestedSpace)).To(Equal(virtualSize + Mi))
-			}
-		}
+	table.DescribeTable("should avoid cleanup before delta copies", func(dataSource DataSourceInterface, expectedCleanup bool) {
+		tmpDir, err := ioutil.TempDir("", "scratch")
+		Expect(err).ToNot(HaveOccurred())
+		defer os.RemoveAll(tmpDir)
+
+		dp := NewDataProcessor(dataSource, "dest", "dataDir", tmpDir, "1G", 0.055, false)
+		Expect(dp.needsDataCleanup).To(Equal(expectedCleanup))
 	},
-		table.Entry("1Mi virtual size, 0 overhead to be 1Mi if <= 1Mi and 2Mi if > 1Mi", Mi, noOverhead),
-		table.Entry("1Mi virtual size, default overhead to be 1Mi if <= 1Mi and 2Mi if > 1Mi", Mi, defaultOverhead),
-		table.Entry("1Mi virtual size, large overhead to be 1Mi if <= 1Mi and 2Mi if > 1Mi", Mi, largeOverhead),
-		table.Entry("40Mi virtual size, 0 overhead to be 40Mi if <= 1Mi and 41Mi if > 40Mi", 40*Mi, noOverhead),
-		table.Entry("40Mi virtual size, default overhead to be 40Mi if <= 1Mi and 41Mi if > 40Mi", 40*Mi, defaultOverhead),
-		table.Entry("40Mi virtual size, large overhead to be 40Mi if <= 40Mi and 41Mi if > 40Mi", 40*Mi, largeOverhead),
-		table.Entry("1Gi virtual size, 0 overhead to be 1Gi if <= 1Gi and 2Gi if > 1Gi", Gi, noOverhead),
-		table.Entry("1Gi virtual size, default overhead to be 1Gi if <= 1Gi and 2Gi if > 1Gi", Gi, defaultOverhead),
-		table.Entry("1Gi virtual size, large overhead to be 1Gi if <= 1Gi and 2Gi if > 1Gi", Gi, largeOverhead),
-		table.Entry("40Gi virtual size, 0 overhead to be 40Gi if <= 1Gi and 41Gi if > 40Gi", 40*Gi, noOverhead),
-		table.Entry("40Gi virtual size, default overhead to be 40Gi if <= 1Gi and 41Gi if > 40Gi", 40*Gi, defaultOverhead),
-		table.Entry("40Gi virtual size, large overhead to be 40Gi if <= 40Gi and 41Gi if > 40Gi", 40*Gi, largeOverhead),
+		table.Entry("ImageIO delta copy", &ImageioDataSource{currentSnapshot: "123", previousSnapshot: "123"}, false),
+		table.Entry("ImageIO base copy", &ImageioDataSource{currentSnapshot: "123", previousSnapshot: ""}, true),
+		table.Entry("VDDK delta copy", &VDDKDataSource{CurrentSnapshot: "123", PreviousSnapshot: "123"}, false),
+		table.Entry("VDDK base copy", &VDDKDataSource{CurrentSnapshot: "123", PreviousSnapshot: ""}, true),
 	)
 })
 
@@ -474,6 +455,53 @@ var _ = Describe("DataProcessorResume", func() {
 	})
 })
 
+var _ = Describe("MergeDelta", func() {
+	It("Should correctly move to merge phase, then rebase and commit", func() {
+		url := &url.URL{}
+		originalBackingFile := "original-backing-file"
+		expectedBackingFile := "rebased-backing-file"
+		originalActualSize := int64(5)
+		expectedActualSize := int64(6)
+
+		mdp := &MockDataProvider{
+			infoResponse:     ProcessingPhaseTransferScratch,
+			transferResponse: ProcessingPhaseMergeDelta,
+			needsScratch:     true,
+			url:              url,
+		}
+
+		dp := NewDataProcessor(mdp, expectedBackingFile, "dataDir", "scratchDataDir", "", 0.055, false)
+		err := errors.New("this operation should not be called")
+		info := &image.ImgInfo{
+			Format:      "",
+			BackingFile: originalBackingFile,
+			VirtualSize: 10,
+			ActualSize:  originalActualSize,
+		}
+		qemuOperations := NewFakeQEMUOperations(err, err, fakeInfoOpRetVal{info, nil}, err, err, nil)
+		replaceQEMUOperations(qemuOperations, func() {
+			// Check original backing file and size before processing
+			info, err := qemuOperations.Info(url)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(info.BackingFile).To(Equal(originalBackingFile))
+			Expect(info.ActualSize).To(Equal(originalActualSize))
+
+			// This should run rebase and commit
+			err = dp.ProcessData()
+			Expect(err).ToNot(HaveOccurred())
+			Expect(2).To(Equal(len(mdp.calledPhases)))
+			Expect(ProcessingPhaseInfo).To(Equal(mdp.calledPhases[0]))
+			Expect(ProcessingPhaseTransferScratch).To(Equal(mdp.calledPhases[1]))
+
+			// Verify backing file was rebased and committed to main data file
+			info, err = qemuOperations.Info(url)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(info.BackingFile).To(Equal(expectedBackingFile))
+			Expect(info.ActualSize).To(Equal(expectedActualSize))
+		})
+	})
+})
+
 func replaceQEMUOperations(replacement image.QEMUOperations, f func()) {
 	orig := qemuOperations
 	if replacement != nil {
@@ -491,7 +519,7 @@ func (o *fakeQEMUOperations) ConvertToRawStream(*url.URL, string, bool) error {
 	return o.e2
 }
 
-func (o *fakeQEMUOperations) Validate(*url.URL, int64, float64) error {
+func (o *fakeQEMUOperations) Validate(*url.URL, int64) error {
 	return o.e5
 }
 
@@ -510,6 +538,24 @@ func (o *fakeQEMUOperations) CreateBlankImage(dest string, size resource.Quantit
 	return o.e6
 }
 
+// Simulate rebase by changing the backing file.
+func (o *fakeQEMUOperations) Rebase(backingFile string, delta string) error {
+	if o.ret4.imgInfo == nil {
+		return errors.New("invalid image info")
+	}
+	o.ret4.imgInfo.BackingFile = backingFile
+	return nil
+}
+
+// Simulate commit by increasing the image size.
+func (o *fakeQEMUOperations) Commit(image string) error {
+	if o.ret4.imgInfo == nil {
+		return errors.New("invalid image info")
+	}
+	o.ret4.imgInfo.ActualSize++
+	return nil
+}
+
 func NewQEMUAllErrors() image.QEMUOperations {
 	err := errors.New("qemu should not be called from this test override with replaceQEMUOperations")
 	return NewFakeQEMUOperations(err, err, fakeInfoOpRetVal{nil, err}, err, err, nil)
